=begin
Component_Edit_Window_v1.0
Copyright 2010, Atelier

Freely use, and abuse at your own risk. Fitness of use is 
not warranty implied.
 
 Select a component or group in order to edit it in a seperate 
 sketchup window (instance). A group will be converted to a
 component after editing. Upon saving or exiting, the second
 window will closed, and the user will be given the option to
 update the model.
  
 v0.01 12/26/2010 Initial beta release edit each instance.
 v0.02 12/27/2010 
 v0.05 12/31/2010 Add edit all component definitions.
 v1.0  01/08/2011 Fix location of saved file bug.
 v1.1  12/03/2011 Glitch in code fixed by TIG.
 v1.2  12/06/2011 More robust checking in File.delete?
 v1.3  12/07/2011 Several aspects of the code tweaked...
				  Temp folder used for temp-copies, MAC friendly path names, 
				  Wrapped in HonoluluDesktop module, extra observers added for 
				  'close'. tidies up temp files better etc.
=end
###
require 'sketchup.rb'
###
module HonoluluDesktop ###
###
### menu
unless file_loaded? File::basename(__FILE__)
	### menu
	UI.add_context_menu_handler{|menu| # add menu to r-click selection menu
		ss=Sketchup.active_model.selection
		if ss[0].is_a?(Sketchup::ComponentInstance)
			sub_menu=menu.add_item("Component Edit Window"){self.edit_instance(ss[0])}
		elsif ss[0].is_a?(Sketchup::Group)
			sub_menu=menu.add_item("Group >> Component + Edit Window"){self.edit_instance(ss[0])}
		end
	}
end
file_loaded File::basename(__FILE__)
###
### observer[s]
class self::ModelObserver < Sketchup::ModelObserver
	def onPreSaveModel(model) 
		ci_string=Sketchup.read_default("honoluludesktop", "ci_edit", "?")
		if model.path.tr("\\","/")==ci_string
			###
			begin
				res=Sketchup.send_action(57665) if not res ###pc only undocumented exit command.
			rescue
				res=Sketchup.send_action('terminate:') if not res #mac only
			end
			begin
				res=Sketchup.send_action('terminate:') if not res #mac only
			rescue
				res=Sketchup.send_action(57665) if not res ###pc only undocumented exit command.
			end
			### reset registry
			Sketchup.write_default("honoluludesktop", "ci_edit", "?")
			###
		end
	end
end
###
class self::AppObserver < Sketchup::AppObserver
    def onNewModel(model)
		ci_string=Sketchup.read_default("honoluludesktop", "ci_edit", "?")
		if model.path.tr("\\","/")==ci_string
			model.add_observer(self::ModelObserver.new)
		end
    end
	def onOpenModel(model)
		ci_string=Sketchup.read_default("honoluludesktop", "ci_edit", "?")
		if model.path.tr("\\","/")==ci_string
			model.add_observer(self::ModelObserver.new)
		end
	end
	def onQuit()
	    ci_string=Sketchup.read_default("honoluludesktop", "ci_edit", "?")
		File.delete(ci_string) if File.exist?(ci_string)
		ci_string=Sketchup.write_default("honoluludesktop", "ci_edit", "?")
	end
end### TIG
# Attach the observer[s].
Sketchup.add_observer(self::AppObserver.new)
Sketchup.active_model.add_observer(self::ModelObserver.new)
###
### set up the editor
def self.edit_instance(instance)###
	model=Sketchup.active_model
	definitions=model.definitions 
	instances=[]
	#dir_name=File.dirname(model.path)
    dir_name=ENV["TMPDIR"] if not dir_name=ENV["TEMP"] ### it's a MAC if not PC
	###
	if instance.is_a?(Sketchup::ComponentInstance)
		odefn=instance.definition
		oname=odefn.name
		instance.definition.instances.each{|e|
			instances << [e.parent.entities, e.transformation, e]
		}
	elsif instance.is_a?(Sketchup::Group)
		oname=instance.name #get group name
		instance.make_unique if instance.entities.parent.instances[1]
		instance=instance.to_component #change to component
		odefn=instance.definition
		odefn.name=oname if not oname.empty? #set component name
		oname=odefn.name
		trans_sel=instance.transformation #save transformation
		instances << [instance.parent.entities, instance.transformation, instance]
	end
	###
	name=(oname.gsub(/[\/\:*?<>|\\]/, '_'))+'-EDIT_WINDOW'
	@comp_name=File.join(dir_name, name+".skp").tr("\\","/")
	Sketchup.write_default("honoluludesktop", "ci_edit", @comp_name)
	### save it
	instance.definition.save_as(@comp_name)
	### reset registry
	### open it
	UI.openURL('file:///'+@comp_name) #open saved file in new sketchup window
	### NB: file:/// is for MAC
	### Now we wait till it's done...
	tid=UI.start_timer(1,true){
	 if Sketchup.read_default("honoluludesktop", "ci_edit", "")=="?"
	    ###
	    UI.stop_timer(tid)
		### check exists
		if File.exist?(@comp_name) and UI.messagebox("Update \""+oname+"\" ?", MB_OKCANCEL) != 2
			###
		  model.start_operation("Component Edit Window 1")
			### erase the originals
			instances.each{|a|
			ents=a[0]
			inst=a[2]
			ents.erase_entities(inst)
			}
			### clear browser - empty the odefn
			odefn.entities.erase_entities(odefn.entities.to_a)
			### so it disappears inside this 'operation'
		  model.commit_operation
			### add edited version back in
		  model.start_operation("Component Edit Window 2")
			defn=definitions.load(@comp_name) #add edited file as a new definition
			defn.name=oname
			### replace the now erased instances with the edited version
			instances.each{|a|
				ents=a[0]
				tran=a[1]
				ents.add_instance(defn, tran)
			}
		  model.commit_operation
	   end#if
		###
		### tidy up temp files, if any skp/skb...
		File.delete(@comp_name) if File.exist?(@comp_name)
		File.delete(@comp_name.gsub(/p$/,'b')) if File.exist?(@comp_name.gsub(/p$/,'b'))
		###
	 end#if
	 ###
	}#timer
	###
end#def

end#module
###
